searchNodes=[{"doc":"Dotenvy is an Elixir implementation of the original dotenv Ruby gem. It is designed to help the development of applications following the principles of the 12-factor app and its recommendation to store configuration in the environment. Unlike other configuration helpers, Dotenvy enforces no convention for the naming of your files: .env is a common choice, you may name your configuration files whatever you wish. See the strategies for examples of various use cases.","ref":"Dotenvy.html","title":"Dotenvy","type":"behaviour"},{"doc":"","ref":"Dotenvy.html#env/3","title":"Dotenvy.env/3","type":"function"},{"doc":"Reads the given system environment variable and converts its value to the given type . Internally, this behaves like System.fetch_env!/1 : it will raise if a variable is not set or if empty values are encounted when non-empty values are required. The conversion is delegated to Dotenvy.Transformer.to!/2 -- see its documentation for a list of supported types. Examples iex&gt; env! ( &quot;PORT&quot; , :integer ) 5432 iex&gt; env! ( &quot;ENABLED&quot; , :boolean ) true","ref":"Dotenvy.html#env!/2","title":"Dotenvy.env!/2","type":"function"},{"doc":"Reads a system environment variable and converts its output or returns a default value. Use of env!/2 is usually recommended over env!/3 because it creates a stronger contract with the environment (i.e. your app literally will not start if required env variables are missing) but there are times where supplying default values is desirable, and the env!/3 function is appropriate for those situations. If the given system environment variable is set , its value is converted to the given type . The provided default value is only used when the system environment variable is not set; the default value is returned as-is, without conversion . This allows greater control of the output. Although this relies on System.fetch_env/1 , it may still raise an error if an unsupported type is provided or if non-empty values are required because the conversion is delegated to Dotenvy.Transformer.to!/2 -- see its documentation for a list of supported types. Examples iex&gt; env! ( &quot;PORT&quot; , :integer , 5432 ) 5433 iex&gt; env! ( &quot;NOT_SET&quot; , :boolean , %{ not : &quot;converted&quot; } ) %{ not : &quot;converted&quot; } iex&gt; System . put_env ( &quot;HOST&quot; , &quot;&quot; ) iex&gt; env! ( &quot;HOST&quot; , :string! , &quot;localhost&quot; ) ** (Dotenvy.Transformer.Error) non-empty value required","ref":"Dotenvy.html#env!/3","title":"Dotenvy.env!/3","type":"function"},{"doc":"A parser implementation should receive the contents read from a file, a map of vars (with string keys, as would come from System.get_env/0 ), and a keyword list of opts . This callback is provided to help facilitate testing. See Dotenvy.Parser for the default implementation.","ref":"Dotenvy.html#c:parse/3","title":"Dotenvy.parse/3","type":"callback"},{"doc":"Like Bash's source command, this loads the given file(s) and sets the corresponding system environment variables using a side effect function ( &amp;System.put_env/1 by default). Files are processed in the order they are given. Values parsed from one file may override values parsed from previous files: the last file parsed has final say. The :overwrite? option determines how the parsed values will be merged with the existing system values. Options :overwrite? boolean indicating whether or not values parsed from provided .env files should overwrite existing system environment variables. It is recommended to keep this false : setting it to true would prevent you from setting variables on the command line, e.g. LOG_LEVEL=debug iex -S mix Default: false :parser module that implements Dotenvy.parse/3 callback. Default: Dotenvy.Parser :require_files specifies which of the given files (if any) must be present. When true , all the listed files must exist. When false , none of the listed files must exist. When some of the files are required and some are optional, provide a list specifying which files are required. If a file listed here is not included in the function's files argument, it is ignored. Default: false :side_effect an arity 1 function called after the successful parsing of each of the given files. The default is &amp;System.put_env/1 , which will have the effect of setting system environment variables based on the results of the file parsing. :vars a map with string keys representing the starting pool of variables. Default: output of System.get_env/0 . Examples iex&gt; Dotenvy . source ( &quot;.env&quot; ) { :ok , %{ &quot;PWD&quot; =&gt; &quot;/users/home&quot; , &quot;DATABASE_URL&quot; =&gt; &quot;postgres://postgres:postgres@localhost/myapp&quot; , # ...etc... } } # If you only want to return the parsed contents of the listed files # ignoring system environment variables altogether iex&gt; Dotenvy . source ( [ &quot;file1&quot; , &quot;file2&quot; ] , side_effect : false , vars : %{ } )","ref":"Dotenvy.html#source/2","title":"Dotenvy.source/2","type":"function"},{"doc":"As source/2 , but returns map on success or raises on error.","ref":"Dotenvy.html#source!/2","title":"Dotenvy.source!/2","type":"function"},{"doc":"This module handles the parsing of the contents of .env files into maps with string keys. See Dotenv File Format for details on the supported file format. This implementation uses parsing over regular expressions for most of its work.","ref":"Dotenvy.Parser.html","title":"Dotenvy.Parser","type":"module"},{"doc":"Parse the given contents , substituting and merging with the given vars .","ref":"Dotenvy.Parser.html#parse/3","title":"Dotenvy.Parser.parse/3","type":"function"},{"doc":"This module provides functionality for converting string values to specific Elixir data types. These conversions were designed to operate on system environment variables, which always store string binaries.","ref":"Dotenvy.Transformer.html","title":"Dotenvy.Transformer","type":"module"},{"doc":"Converts strings into Elixir data types with support for nil-able values. Each type determines how to interpret the incoming string, e.g. when the type is :integer , an empty string is considered a 0 ; when :integer? is the type , and empty string is converted to nil . Remember: Use a ? suffix when an empty string should be considered nil (a.k.a. a &quot;nullable&quot; value). Use a ! suffix when an empty string is not allowed. Use this when values are required. Types The following types are supported: :atom - converts to an atom. An empty string will be the atom :&quot;&quot; (!). :atom? - converts to an atom. An empty string will be considered nil :atom! - converts to an atom. An empty string will raise. :boolean - &quot;false&quot;, &quot;0&quot;, or an empty string &quot;&quot; will be considered boolean false . Any other non-empty value is considered true . :boolean? - as above, except an empty string will be considered nil :boolean! - as above, except an empty string will raise. :charlist - converts string to charlist. :charlist? - converts string to charlist. Empty string will be considered nil . :charlist! - as above, but an empty string will raise. :integer - converts a string to an integer. An empty string will be considered 0 . :integer? - as above, but an empty string will be considered nil . :integer! - as above, but an empty string will raise. :float - converts a string to an float. An empty string will be considered 0 . :float? - as above, but an empty string will be considered nil . :float! - as above, but an empty string will raise. :existing_atom - converts into an existing atom. Raises error if the atom does not exist. :existing_atom? - as above, but an empty string will be considered nil . :existing_atom! - as above, but an empty string will raise. :module - converts a string into an Elixir module name. Raises on error. :module? - as above, but an empty string will be considered nil . :module! - as above, but an empty string will raise. :string - no conversion (default) :string? - empty strings will be considered nil . :string! - as above, but an empty string will raise. Examples iex&gt; to! ( &quot;debug&quot; , :atom ) :debug iex&gt; to! ( &quot;&quot; , :boolean ) false iex&gt; to! ( &quot;&quot; , :boolean? ) nil iex&gt; to! ( &quot;5432&quot; , :integer ) 5432","ref":"Dotenvy.Transformer.html#to!/2","title":"Dotenvy.Transformer.to!/2","type":"function"},{"doc":"Dotenvy Dotenvy is an Elixir implementation of the original dotenv Reby gem, compatible with mix and releases . It is designed to help the development of applications following the principles of the 12-factor app and its recommendation to store configuration in the environment.","ref":"readme.html","title":"Dotenvy","type":"extras"},{"doc":"Add dotenvy to your list of dependencies in mix.exs : def deps do [ { :dotenvy , &quot;~&gt; 0.3.0&quot; } ] end It has no dependencies.","ref":"readme.html#installation","title":"Dotenvy - Installation","type":"extras"},{"doc":"Dotenvy is designed to help configure your application at runtime , and one of the most effective places to do that is inside config/runtime.exs (available since Elixir v1.11). The Dotenvy.source/2 function can accept a single file or a list of files. When combined with Config.config_env/0 it is easy to load up environment-specifc config, e.g. source ( [ &quot;.env&quot; , &quot;.env. \#{ config_env ( ) } &quot; , &quot;.env. \#{ config_env ( ) } .local&quot; ] ) By default, the listed files do not need to exist -- the function only needs to know where to look. This makes it easy to commit default values while still leaving the door open to developers to override values via their own configuration files. Unlike other packages, Dotenvy has no opinions about the names or locations of your dotenv config files, you just need to pass their paths to Dotenvy.source/2 or Dotenvy.source!/2 . For a simple example, we can load a single file: # config/runtime.exs import Config import Dotenvy source! ( &quot;.env&quot; ) config :myapp , MyApp.Repo , database : env! ( &quot;DATABASE&quot; , :string! ) , username : env! ( &quot;USERNAME&quot; , :string ) , password : env! ( &quot;PASSWORD&quot; , :string ) , hostname : env! ( &quot;HOSTNAME&quot; , :string! ) , pool_size : env! ( &quot;POOL_SIZE&quot; , :integer ) , adapter : env! ( &quot;ADAPTER&quot; , :module , Ecto.Adapters.Postgres ) , pool : env! ( &quot;POOL&quot; , :module? ) And then define your variables in the file(s) to be sourced. Dotenvy has no opinions about what you name your files; .env is merely a convention. # .env DATABASE = myapp_dev USERNAME = myuser PASSWORD = mypassword HOSTNAME = localhost POOL_SIZE = 10 POOL = When you set up your application configuration in this way, you are creating a contract with the environment (errors will be raised if certain system variables are not set because Dotenvy.env!/2 relies on System.fetch_env!/1 ), and this is an approach that works equally well for your day-to-day development and testing, as well as for mix releases. Read the configuration strategies for more detailed examples of how to configure your app. Refer to the &quot;dotenv&quot; ( .env ) file format for more examples and features of the supported syntax. See the Dotenvy module documentation on its functions. Note for Mix Tasks If you have authored your own Mix tasks, you must ensure that they load the application configuration in a way that is compatible with the runtime config. A good way to do this is to include Mix.Task.run(&quot;app.config&quot;) inside the run/1 implementation, e.g. def run ( _args ) do Mix.Task . run ( &quot;app.config&quot; ) # ... end If you are dealing with third-party mix tasks that fail to properly load configuration, you may need to manually call mix app.config before running them, e.g. mix do app.config other.task Image Attribution: &quot;dot&quot; by Stepan Voevodin from the Noun Project","ref":"readme.html#usage","title":"Dotenvy - Usage","type":"extras"},{"doc":"Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Initial release.","ref":"changelog.html#v0-1-0","title":"Changelog - v0.1.0","type":"extras"},{"doc":"Adds support for default type of :string to the Dotenvy.env!/2 and Dotenvy.env/3 functions.","ref":"changelog.html#v0-2-0","title":"Changelog - v0.2.0","type":"extras"},{"doc":"Renames Dotenvy.Transformer.to/2 to Dotenvy.Transformer.to!/2 to better communicate that it may raise an error. Returns key name in errors for easier troubleshooting. Tracks an error if the :require_files option lists a file not included in the files input (for sanity). Introduces Dotenvy.env!/3 (which is the same as Dotenvy.env/3 but with no defaults provided). This better communicates that it may raise an error (because internally it relies on Dotenvy.Transformer.to!/2 ) Deprecates Dotenvy.env/3 in favor of Dotenvy.env!/3","ref":"changelog.html#v0-3-0","title":"Changelog - v0.3.0","type":"extras"},{"doc":"Dotenv File Format .env files (a.k.a. &quot;dotenv&quot;) store key-value pairs in a format descended from simple bash files that exported environment variables. This implementation cleaves closely to the format described by the original dotenv package, but it is not a direct match (by design). Typically, a dotenv ( .env ) file is formatted into simple key-value pairs: S3_BUCKET = YOURS3BUCKET SECRET_KEY = YOURSECRETKEYGOESHERE You may add export in front of each line so you can source the file in bash (see Dotenvy.source/2 ): export S3_BUCKET = YOURS3BUCKET export SECRET_KEY = YOURSECRETKEYGOESHERE","ref":"dotenv-file-format.html","title":"Dotenv File Format","type":"extras"},{"doc":"For the sake of portability (and sanity), environment variable names must consist solely of letters, digits, and the underscore ( _ ) and must not begin with a digit. In regex-speak: [ a - zA - Z_ ] + [ a - zA - Z0 - 9 _ ] * Example variable names DATABASE_URL # ok foobar # ok NO - WORK # &lt;-- invalid !!! Ü BER # &lt;-- invalid !!! 2 MUCH # &lt;-- invalid !!!","ref":"dotenv-file-format.html#variable-names","title":"Dotenv File Format - Variable Names","type":"extras"},{"doc":"Values are to the right of the equals sign. They may be quoted. Using single quotes will prevent variables from being interpolated. SIMPLE = xyz123 INTERPOLATED = &quot;Multiple \\n Lines and variable substitution: ${SIMPLE}&quot; NON_INTERPOLATED = &#39;raw text without variable interpolation&#39; MULTILINE = &quot;&quot;&quot; long text here, e.g. a private SSH key &quot;&quot;&quot;","ref":"dotenv-file-format.html#values","title":"Dotenv File Format - Values","type":"extras"},{"doc":"The following character strings will be interpreted (i.e. escaped) as specific codepoints in the same way you would expect if the values were assigned inside a script. Remember: when a text file is read, it is read as a series of utf8 encoded code points. Character sequences like \\n have no special meaning until they are &quot;escaped&quot; and a combination of codepoints is replaced by a single codepoint. \\n Linefeed (aka newline); &lt;&lt;92, 110&gt;&gt; -&gt; &lt;&lt;10&gt;&gt; \\r Carriage return; &lt;&lt;92, 114&gt;&gt; -&gt; &lt;&lt;13&gt;&gt; \\t Tab; -&gt; &lt;&lt;92, 116&gt;&gt; -&gt; &lt;&lt;9&gt;&gt; \\f Form feed; -&gt; &lt;&lt;92, 102&gt;&gt; -&gt; &lt;&lt;12&gt;&gt; \\b Backspace; -&gt; &lt;&lt;92, 98&gt;&gt; -&gt; &lt;&lt;8&gt;&gt; \\&quot; Double-quote; -&gt; &lt;&lt;92, 34&gt;&gt; -&gt; &lt;&lt;34&gt;&gt; \\' Single-quote; -&gt; &lt;&lt;92, 39&gt;&gt; -&gt; &lt;&lt;39&gt;&gt; \\\\ Backslash; -&gt; &lt;&lt;92, 92&gt;&gt; -&gt; &lt;&lt;92&gt;&gt; \\uFFFF Unicode escape (4 hex characters to denote the codepoint) If a backslash precedes any other character, that character will be interpretted literally: i.e. the backslash will be ignored and removed from output. Interpolation (a.k.a. Variable Substitution) Values left unquoted or wrapped in double-quotes will interpolate variables in the ${VAR} syntax. This can be useful for referencing existing system environment variables or to reference varaibles previously parsed. For example: USER = admin EMAIL = $ { USER } @example . org DATABASE_URL = &quot;postgres://${USER}@localhost/my_database&quot; CACHE_DIR = $ { PWD } / cache Multi-line values (e.g. private keys) can use the triple-quoted heredoc syntax: PRIVATE_KEY = &quot;&quot;&quot; -----BEGIN RSA PRIVATE KEY----- ... HkVN9... ... -----END DSA PRIVATE KEY----- &quot;&quot;&quot; Non-Interpolated If your values must retain ${} in their output, wrap the value in single quotes, e.g.: PASSWORD = &#39;!@G0${k}k&#39; MESSAGE_TEMPLATE = &#39;&#39;&#39; Hello ${PERSON}, Nice to meet you! &#39;&#39;&#39;","ref":"dotenv-file-format.html#escape-sequences","title":"Dotenv File Format - Escape Sequences","type":"extras"},{"doc":"The hash-tag # symbol denotes a comment when on its own line or when it follows a quoted value. It is not treated as a comment when it appears within quotes. # This is a comment SECRET_KEY = YOURSECRETKEYGOESHERE # also a comment SECRET_HASH = &quot;something-with-a-hash-#-this-is-not-a-comment&quot;","ref":"dotenv-file-format.html#comments","title":"Dotenv File Format - Comments","type":"extras"},{"doc":"Strategies Although there are other places where Dotenvy may prove useful, it was designed with the config/runtime.exs in mind: most of the following use-cases will focus on that because it offers a clean and declarative way to load up the necessary variables.","ref":"strategies.html","title":"Strategies","type":"extras"},{"doc":"Configuration providers are most often invoked in the context of releases, and although they can solve certain problems that arise in production deployments, they tend to be an awkward fit for regular day-to-day development. Dotenvy seeks to normalize how configuration is loaded across environments, so having different methods depending on how you run your app is antithetical. We do not want some code that runs only in certain environments and not in others: it can make for untested or untestable code. Secondly, configuration providers sometimes shift the task of &quot;shaping&quot; the configuration out of Elixir and into some static representation (e.g. JSON or TOML). The allure of a straight-forward static file is deceiving because there is no easy way to delineate Elixir-specific subtleties such as distinguishing between keyword lists and maps. When configuration providers &quot;solve&quot; one problem, they often create another: it can require some busywork to convert values back into Elixir variable types that your application requires. For these reasons, Dotenvy does not rely on configuration providers ; dotenv files are an easier &quot;lingua franca&quot;.","ref":"strategies.html#a-note-on-configuration-providers","title":"Strategies - A Note on Configuration Providers","type":"extras"},{"doc":"The distinctions between &quot;dev&quot; and &quot;prod&quot; become less clear when we focus on configuration: ideally, the app is the same in all environments, it is only the configuration values themselves that can be described as &quot;dev&quot; or &quot;prod&quot; -- in this example they will live inside a .env file. Let's look at the three files that will make this work: config/config.exs # compile-time config import Config config :myapp , ecto_repos : [ MyApp.Repo ] config :myapp , MyApp.Repo , migration_timestamps : [ type : :utc_datetime , inserted_at : :created_at ] config/runtime.exs import Config import Dotenvy source ( &quot;.env&quot; ) if config_env ( ) == &quot;test&quot; do config :myapp , MyApp.Repo , database : &quot;myapp_test&quot; , username : &quot;test-user&quot; , password : &quot;test-password&quot; , hostname : &quot;localhost&quot; , pool_size : 10 , adapter : Ecto.Adapters.Postgres , pool : Ecto.Adapters.SQL.Sandbox else config :myapp , MyApp.Repo , database : env! ( &quot;DATABASE&quot; , :string! ) , username : env! ( &quot;USERNAME&quot; , :string ) , password : env! ( &quot;PASSWORD&quot; , :string ) , hostname : env! ( &quot;HOSTNAME&quot; , :string! ) , pool_size : env! ( &quot;POOL_SIZE&quot; , :integer ) , adapter : env ( &quot;ADAPTER&quot; , :module , Ecto.Adapters.Postgres ) , pool : env! ( &quot;POOL&quot; , :module? ) end .env (dev or prod) DATABASE = myapp_dev USERNAME = myuser PASSWORD = mypassword HOSTNAME = localhost POOL_SIZE = 10 POOL = The .env shows some values suitable local development; if the app were deployed on a production box, it would be the same shape, but its values would point to a production database. For tests, values are hard-coded inside runtime.exs . This is one admittedly heavy-handed way to ensure that your test runs don't accidentally hit the wrong database, but it does mean that there is a small block of untestable code inside the if-statement. You may notice that in this example we have done away with config/dev.exs , config/test.exs , and config/prod.exs . These should be used only when your app has a legitimate compile-time need. If you can configure something at runtime, you should configure it at runtime. These extra config files are omitted to help demonstrate how the decisions about how the app should run can often be pushed into runtime.exs . This should help avoid confusion that often arises between compile-time and runtime configuration.","ref":"strategies.html#dotenv-for-dev-and-prod","title":"Strategies - Dotenv for Dev and Prod","type":"extras"},{"doc":"It is possible to use only a config.exs and a runtime.exs file to configure many Elixir applications: let the .env tell the app how to run! Consider the following setup: config/config.exs # compile-time config import Config config :myapp , ecto_repos : [ MyApp.Repo ] config :myapp , MyApp.Repo , migration_timestamps : [ type : :utc_datetime , inserted_at : :created_at ] config/runtime.exs import Config import Dotenvy source ( [ &quot;.env&quot; , &quot;.env. \\# {config_env()}&quot; ] ) config :myapp , MyApp.Repo , database : env! ( &quot;DATABASE&quot; , :string! ) , username : env! ( &quot;USERNAME&quot; , :string ) , password : env! ( &quot;PASSWORD&quot; , :string ) , hostname : env! ( &quot;HOSTNAME&quot; , :string! ) , pool_size : env! ( &quot;POOL_SIZE&quot; , :integer ) , adapter : env ( &quot;ADAPTER&quot; , :module , Ecto.Adapters.Postgres ) , pool : env! ( &quot;POOL&quot; , :module? ) .env (dev or prod) DATABASE = myapp_dev USERNAME = myuser PASSWORD = mypassword HOSTNAME = localhost POOL_SIZE = 10 POOL = .env.test DATABASE = myapp_test USERNAME = myuser PASSWORD = mypassword HOSTNAME = localhost POOL_SIZE = 10 POOL = Ecto.Adapters.SQL.Sandbox The above setup would likely commit the .env.test file so it was sure to override, and add .env to .gitignore , but other strategies are possible. The above example demonstrates developer settings appropriate for local development in the sample .env file, but a production deployment would only differ in its values : the shape of the file would be the same. The .env.test file is loaded when running tests, so its values override any of the values set in the .env , but they would not override any pre-existing system variables because the :overwrite? option is false by default. By using Dotenvy.env!/2 , a strong contract is created with the environment: the system running this app must have the designated environment variables set somehow, otherwise this app will not start (and a specific error will be raised). Using the nil-able variants of the type-casting (those ending with ? ) is an easy way to fall back to nil when the variable contains an empty string: env!(&quot;POOL&quot;, :module?) requires that the POOL variable is set, but it will return a nil if the value is an empty string. See Dotenvy.Transformer for more details.","ref":"strategies.html#dotenvs-for-all-environments","title":"Strategies - Dotenvs for All Environments","type":"extras"}]